// Uwe Zimmermann
// 2013-12-17
/*
   LED PWM with remote control

*/

/* pinout ATtiny25/45/85

/Reset     PB5|1    8|Vcc
 IR        PB3|2    7|PB2 SCK
 LED2 OC1B PB4|3    6|PB1 MISO OC0B PWM
           Gnd|4    5|PB0 MOSI OC0A 

*/

#define F_CPU 8000000UL  // 8 MHz internal

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "irmp.h"

// chinese multicolor LED remote
#define LEDPROTOCOL 2
#define LEDADDRESS  65280
#define LEDUP       92
#define LEDDOWN     93
#define LEDONOFF    64 

#define PWMTOP 63
#define PWM OCR0B

#define PWM2TOP 20
#define PWM2 OCR1B

#define TIMER_FADE_STEP 1500

volatile uint16_t timer;
enum {FADE_UP, FADE_DOWN, FADE_STOP};

uint8_t LED2_fade = FADE_UP;      
uint8_t LED_fade  = FADE_STOP;      


void timer1_init (void)
{
#if defined (__AVR_ATtiny45__) || defined (__AVR_ATtiny25__) || defined (__AVR_ATtiny85__) 
               // ATtiny45 / ATtiny85:
#if F_CPU >= 16000000L
    OCR1C   =  (F_CPU / F_INTERRUPTS / 8) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 8
    TCCR1   = (1 << CTC1) | (1 << CS12);                                    // switch CTC Mode on, set prescaler to 8
#else
    OCR1C   =  (F_CPU / F_INTERRUPTS / 4) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 4
    TCCR1   = (1 << CTC1) | (1 << CS11) | (1 << CS10);                      // switch CTC Mode on, set prescaler to 4
#endif

#else                                                                       // ATmegaXX:
    OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                  // compare value: 1/15000 of CPU frequency
    TCCR1B  = (1 << WGM12) | (1 << CS10);                                   // switch CTC Mode on, set prescaler to 1
#endif

#ifdef TIMSK1
    TIMSK1  = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#else
    TIMSK   = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#endif
}

#ifdef TIM1_COMPA_vect                                                      // ATtiny84
#define COMPA_VECT  TIM1_COMPA_vect
#else
#define COMPA_VECT  TIMER1_COMPA_vect                                       // ATmega
#endif

ISR(COMPA_VECT)                                                             // Timer1 output compare A interrupt service routine, called every 1/15000 sec
{
  (void) irmp_ISR();                                                        // call irmp ISR
  // call other timer interrupt routines...
  timer ++;
}

void main_init(void)
{
  // timer0 PWM, phase correct
  PWM    = 0;
  OCR0A  = PWMTOP;
  TCCR0A = (0 << COM0A1) | (0 << COM0A0) 
         | (1 << COM0B1) | (0 << COM0B0)
         | (0 << WGM01)  | (1 << WGM00);
  TCCR0B = (1 << WGM02)
         | (0 << CS02) | (0 << CS01) | (1 << CS00);
  DDRB  |= (1 << PB1);  // PWM OC0B
  DDRB  |= (1 << PB4);  // PWM OC1B
  PORTB |= (1 << PB3); // pull-up IR
  irmp_init();                                                            // initialize irmp
  timer1_init();

  PWM2   = 0;
  GTCCR |= (1 << PWM1B) | (1 << COM1B1) | (0 << COM1B0);

  timer = 0;
  sei();
}

//
// handle fading of main and aux LED
//
void fade(void)
{
  if (timer > TIMER_FADE_STEP)
  {
    timer = 0;
    switch (LED2_fade)
    {
      case FADE_UP:
        if (PWM2 < PWM2TOP)
        { 
          PWM2 ++;
        }
        else
        {
          LED2_fade = FADE_DOWN;
        }
        break;
      case FADE_DOWN:
        if (PWM2 > 1)
        { 
          PWM2 --;
        }
        else
        {
          LED2_fade = FADE_UP;
        }
        break;
      default:
        break;
    } //end switch
 
   switch (LED_fade)
    {
      case FADE_UP:
        if (PWM < PWMTOP)
        { 
          PWM ++;
        }
        else
        {
          LED_fade = FADE_STOP;
        }
        break;
      case FADE_DOWN:
        if (PWM > 0)
        { 
          PWM --;
        }
        else
        {
          LED_fade = FADE_STOP;
          LED2_fade = FADE_UP;
        }
        break;
      default:
        break;
    } //end switch
  }
}


int main(void)
{
  IRMP_DATA irmp_data;
  main_init();
  LED_fade = FADE_UP;
  LED2_fade = FADE_STOP;


  while(1) 
  {
    // handle fading
    fade();

    if (irmp_get_data (&irmp_data))
    {
      if ((irmp_data.protocol==LEDPROTOCOL) && (irmp_data.address==LEDADDRESS))
      {
        OCR1B = 0;
        LED2_fade = FADE_STOP;
        switch (irmp_data.command)
        {
          case LEDONOFF:
            if ((irmp_data.flags & IRMP_FLAG_REPETITION)==0)
            {
              if (PWM > 0)
              {
                LED_fade = FADE_DOWN;
              }
              else
              {
                LED_fade = FADE_UP;
              }
            } 
            break;
          case LEDUP:
            LED_fade = FADE_STOP;
            if (PWM < PWMTOP)
            {
              PWM ++;
            }
            break;
          case LEDDOWN:
            LED_fade = FADE_STOP;
            if (PWM > 0)
            {
              PWM --;
            }
            else
            {
              LED2_fade = FADE_UP;
            }
            break;
          default:
            break;
        }
      }
    }
  }
  return 0;
}


